# 1 Introduction

Depexplorer is a powerful command-line tool for exploring package dependencies on Linux systems. It provides various options to query and visualize the dependency structure of a given package.



# 2 Deployment

1 Install rpm-dep

```
sudo dnf install rpm-dep
```



2 Edit `/etc/rpm-dep/dep-repodata.conf` to set repodata configuration file

```
cat /etc/rpm-dep/dep-repodata.conf 
[repo_name]                                                                                                               
name: tss 

[src_repo_paths]
path1: http://xxx/bin_repo/Packages/repodata
...

[bin_repo_paths]
path1: http://xxx/src_repo/Packages/repodata
...
```

The configuration file defaults to the repodata source, and you can add more if needed.



3 Start redis service

```
sudo systemctl start redis
```



4 Start rpm-depd service

```
sudo systemctl start rpm-depd
```

The service startup takes some time.


5 (Optional) Start the refresh-rpm-depd scheduled service and refresh rpm-depd
```
sudo systemctl enable refresh-rpm-depd.timer
sudo systemctl start refresh-rpm-depd.timer
```


# 3 User Guide

## 3.1 Command Usage

### 3.1.1 Query Installation Dependencies

The command to query installation dependencies is as follows:

```
rpm-dep -i|--install \$pkg_name [-l|--level \$level] [-o|--output \$output_path] [-t|--typeshow {graph,json,all}] [-s|--strip]
```

The contents in `[]` are all optional.

```
	-l|--level \$level                   Specify the installation level, default is 2 levels
	-o|--output \$output_path            Specify the output path, default is the current directory
	-t|--typeshow {graph,json,all}  Specify the output type, default is json format, graph is currently not supported
	-s|--strip                           Remove packages that do not provide library capabilities
	--src                                The input parameter is the source package name, and the runtime dependency is checked with the source package as the granularity
	--only-strong                        only show strong dependency
```

**Note:**

**1 By default, the input is the binary package name
2 Do not add version number, .rpm, etc. to the input**

### 3.1.2 Query Installed Dependencies

The command to query installed dependencies is as follows:

```
rpm-dep -I|--install_by \$pkg_name [-l|--level \$level] [-o|--output \$output_path] [-t|--typeshow {graph,json,all}] [-s|--strip]
```

The contents in `[]` are all optional.

```
	-l|--level \$level                   Specify the installation level, default is 2 levels
	-o|--output \$output_path            Specify the output path, default is the current directory
	-t|--typeshow {graph,json,all}  Specify the output type, default is json format, graph is currently not supported
	-s|--strip                           Remove packages that do not provide library capabilities
	--src                                The input parameter is the source package name, and the runtime dependency is checked with the source package as the granularity
	--only-strong                        only show strong dependency
```

**Note:**

**1 By default, the input is the binary package name
2 Do not add version number, .rpm, etc. to the input**

### 3.1.3 Query Compilation Dependencies

The command to query compilation dependencies is as follows:

```
rpm-dep -b|--build \$pkg_name [-l|--level \$level] [-o|--output \$output_path] [-t|--typeshow {graph,json,all}] [-s|--strip]
```

The contents in `[]` are all optional.

```
	-l|--level \$level                   Specify the installation level, default is 2 levels
	-o|--output \$output_path            Specify the output path, default is the current directory
	-t|--typeshow {graph,json,all}  Specify the output type, default is json format, graph is currently not supported
	-s|--strip                           Remove packages that do not provide library capabilities
	--src                                The input parameter is the source package name, and the runtime dependency is checked with the source package as the granularity
	--only-strong                        only show strong dependency
```

**Note:**

**1 By default, the input is the binary package name
2 Do not add version number, .rpm, etc. to the input
3 No matter whether --src is added or not, the result of the query is to query the dependency with the source package where the input parameter is located as the target, because the specific compilation dependency of a certain subpackage cannot be known**
	--only-strong                        only show strong dependency

### 3.1.4 Query Compilation Dependencies

The command is as follows:

```
rpm-dep -B|--build_by \$pkg_name [-l|--level \$level] [-o|--output \$output_path] [-t|--typeshow {graph,json,all}] [-s|--strip]

```

The contents in `[]` are all optional.

```
	-l|--level \$level                   Specify the installation level, default is 2 levels
	-o|--output \$output_path            Specify the output path, default is the current directory
	-t|--typeshow {graph,json,all}  Specify the output type, default is json format, graph is currently not supported
	-s|--strip                           Remove packages that do not provide library capabilities
	--src                                The input parameter is the source package name, and the runtime dependency is checked with the source package as the granularity
	--only-strong                        only show strong dependency
```

**Note:**

**1 By default, the input is the binary package name
2 Do not add version number, .rpm, etc. to the input**


### 3.1.5 Query Compilation Environment Dependencies

The command to query compilation environment dependencies is as follows:

```
rpm-dep -e|--environ \$pkg_name [-l|--level \$level] [-o|--output \$output_path] [-t|--typeshow {graph,json,all}] [-s|--strip]
```

The contents in `[]` are all optional.

```
	-l|--level \$level                   Specify the installation level, default is 2 levels
	-o|--output \$output_path            Specify the output path, default is the current directory
	-t|--typeshow {graph,json,all}  Specify the output type, default is json format, graph is currently not supported
	-s|--strip                           Remove packages that do not provide library capabilities
	--src                                The input parameter is the source package name, and the runtime dependency is checked with the source package as the granularity
```

**Note:**

**1 By default, the input is the binary package name
2 Do not add version number, .rpm, etc. to the input**

### 3.1.6 Query Compilation and Install Dependencies

The command is as follows:

```
rpm-dep -r|--requires \$pkg_name [-l|--level \$level] [-o|--output \$output_path] [-t|--typeshow {graph,json,all}] [-s|--strip]

```

The contents in `[]` are all optional.

```
	-l|--level \$level                   Specify the installation level, default is 2 levels
	-o|--output \$output_path            Specify the output path, default is the current directory
	-t|--typeshow {graph,json,all}  Specify the output type, default is json format, graph is currently not supported
	-s|--strip                           Remove packages that do not provide library capabilities
	--src                                The input parameter is the source package name, and the runtime dependency is checked with the source package as the granularity
```

**Note:**

**1 By default, the input is the source package name
2 Do not add version number, .rpm, etc. to the input**

### 3.1.7 Reinitialize rpm-depd

If the user modifies `/etc/rpm-dep/dep-repodata.conf`, it is necessary to reinitialize the rpm-depd service, which can be done by executing the following command:

```
rpm-dep --init
```

### 3.1.8 Batch Processing Command
Usage scenarios
1. When counting the dependencies of a batch of source packages, the query efficiency of rpm-dep through the binary package entry is relatively low
2. For the dependent situation, users need to pay attention to the situation where both installation and compilation are considered comprehensively
3. Query which subpackages a src package has
4. Query the compilation dependencies of a src package

Command format:
Count the installation and compilation dependencies of a source package
```
ci-rpm-dep -R|--whatrequire \$pkg_name [-o|--output \$output_path]
```

Count which subpackages a source package has
```
ci-rpm-dep -S|--subpkgs \$pkg_name
```

The contents in `[]` are all optional

```
	-o|--output \$output_path            Specify the output path, default is the current directory, output both installation and compilation dependencies json files at the same time
```
Count the compilation dependencies of a source package
```
ci-rpm-dep -r|--requires \$pkg_name [-o|--output \$output_path]
```

**Note:**

**1 Input is the source package name
2 Do not add version number, .rpm, etc. to the input
3 Do not support specifying level, default display to level 2**

Query whether the current source package provides library capabilities
```
ci-rpm-dep -l|----islibs \$pkg_name]
```
The format is as follows:
```
# ci-rpm-dep -l glibc
[INFO] libs found in your pkg: glibc

# ci-rpm-dep -l make
[WARN] libs not found in your pkg: make
```

**Note:**

**1 Input is the source package name
2 Do not add version number, .rpm, etc. to the input**

### 3.1.9 rpm-depso Command
rpm-depso command is an improved version of rpm-dep command. According to the user-specified package name and `.so / .a` file name, it can list the installed dependencies from the granularity of `.so / .a`. The usage is as follows:

Query the installed dependencies of `$so_name`
```
rpm-depso -I|--install_by $bin_name $so_name [-l|--level \$level] [-o|--output \$output_path] [-s|--strip]
```
**Note:**

**1 Input is the source package name
2 Do not add version number, .rpm, etc. to the input
3 Default output is json file
4 It is recommended to choose the so name without version suffix in the devel package
5 rpm-dep obtains dependency relationships from repodata, the latter's so information is incomplete and still under analysis, at this stage if so is not found, the rpm-depso command degrades to rpm-dep -I command and throws a warning**

### 3.1.10 rpm-unfold Command
The rpm-unfold command is used to list the dependency order of source packages in batches, providing guidance for CI builds.

```
rpm-unfold [--clean] [--debug] [--quiet] [--install | --build | --build_by] [--level LEVEL] [--outpath OUTPATH] [--help]
                  [--version] [--strip]
                  pkg_name [pkg_name ...]

-q|--quiet    Do not print log information
-l|--level    Search levels, default is to search 3 levels and output 2 levels
-s|--strip    Exclude packages that do not provide library capabilities
-c|--clean    During the search, the rpm-dep-private-$hashid directory will be generated under /var/tmp/, clean up this directory, default is not to clean
-i|--install  Sort installation dependencies, input is the source package name
-b|--build    Sort compilation dependencies, input is the source package name
-B|--build_by Sort dependencies that are compiled, input is the source package name
```

### 3.1.11 rpm-bundled command
The rpm-bundled command is used to count provides bundled packages
```
usage: rpm-bundled [--all | --src | --bin] [--debug] [--outpath OUTPATH] [--help] [--version] [package_name]

A tool for list bundled messages in repodata

positional arguments:
  package_name Package name

options:
  --all, -a                      displays all provides bundled src packages in repodata and their corresponding bundled symbols
  --src, -s                      displays the specified src package provides bundled sub-packages and their corresponding bundled symbols
  --bin, -b                      displays the specified bin package provides bundled
  --debug, -d                    debugging
  --outpath OUTPATH, -o OUTPATH  command log output path for --all option
```

### 3.1.11 Debug Command
The rpm-dep series of tools have added log output points on the key paths of the code. By enabling the debug option, these debug logs can be printed to facilitate problem positioning. To enable debug, simply add the following option when executing the above command:
```
--debug | -d
```

## 3.2 Output Files

### 3.2.1 json Format

We take the following command as an example:

```
rpm-dep -i python3-flask
```



The json file is as follows, with the pkg_name display format being "bin name (src name)", and next indicating the next level:

```
{
  "level": 1,
  "pkg_name": "python3-flask (python-flask)",
  "next": [
    {
      "level": 2,
      "pkg_name": "python3-werkzeug (python-werkzeug)",
      "next": []
    },
    {
      "level": 2,
      "pkg_name": "python3-jinja2 (python-jinja2)",
      "next": []
    },
    {
      "level": 2,
      "pkg_name": "python3-itsdangerous (python-itsdangerous)",
      "next": []
    },
    {
      "level": 2,
      "pkg_name": "python3-click (python-click)",
      "next": []
    },
    {
      "level": 2,
      "pkg_name": "python3 (python3.10)",
      "next": []
    }
  ]
}
```



### 3.2.2 yaml Format

We take the following command as an example:

```
rpm-dep -i python3-flask
```



The yaml file is as follows, with the pkg_name display format being "bin name (src name)":

```
---
- level: 1
  pkg_name: python3-flask (python-flask)
- - level: 2
    pkg_name: python3-werkzeug (python-werkzeug)
- - level: 2
    pkg_name: python3-jinja2 (python-jinja2)
- - level: 2
    pkg_name: python3-itsdangerous (python-itsdangerous)
- - level: 2
    pkg_name: python3-click (python-click)
- - level: 2
    pkg_name: python3 (python3.10)
```
